@@ -1,13 +1,18 @@
 import asyncio
-import re
 from datetime import datetime
 from pathlib import Path
-from typing import Optional, Set, Tuple
+from typing import Optional, Tuple
 
-import fastapi
 from fastapi import APIRouter, Depends, WebSocketDisconnect
 from fps.hooks import register_router  # type: ignore
 from fps_contents.routes import read_content, write_content  # type: ignore
+
+try:
+    from fps_contents.watchfiles import awatch
+
+    has_awatch = True
+except ImportError:
+    has_awatch = False
 from jupyter_ydoc import ydocs as YDOCS  # type: ignore
 from ypy_websocket.websocket_server import WebsocketServer, YRoom  # type: ignore
 from ypy_websocket.ystore import BaseYStore, SQLiteYStore, YDocNotFound  # type: ignore
@@ -27,14 +32,6 @@ class JupyterSQLiteYStore(SQLiteYStore):
 router = APIRouter()
 
 
-def get_path_param_names(path: str) -> Set[str]:
-    return {name.split(":")[0] for name in re.findall("{(.*?)}", path)}
-
-
-# FIXME: remove the patch when https://github.com/tiangolo/fastapi/pull/3879 is merged
-fastapi.utils.get_path_param_names.__code__ = get_path_param_names.__code__
-
-
 def to_datetime(iso_date: str) -> datetime:
     return datetime.fromisoformat(iso_date.rstrip("Z"))
 
@@ -216,13 +213,19 @@ class YDocWebSocketHandler:
         return skip
 
     async def watch_file(self):
-        poll_interval = 1  # FIXME: pass in config
-        if not poll_interval:
-            self.room.watcher = None
-            return
-        while True:
-            await asyncio.sleep(poll_interval)
-            await self.maybe_load_document()
+        if has_awatch:
+            file_format, file_type, file_path = self.get_file_info()
+            async for changes in awatch(file_path):
+                await self.maybe_load_document()
+        else:
+            # contents plugin doesn't provide watcher, fall back to polling
+            poll_interval = 1  # FIXME: pass in config
+            if not poll_interval:
+                self.room.watcher = None
+                return
+            while True:
+                await asyncio.sleep(poll_interval)
+                await self.maybe_load_document()
 
     async def maybe_load_document(self):
         file_format, file_type, file_path = self.get_file_info()
